# ANN Benchmarks

## 测试方法

1. 安装依赖库

    ```bash
    pip3 install -r requirements.txt --user
    ```

2. 安装tair-py

    ```bash
    git clone https://github.com/alibaba/tair-py.git
    cd tair-py
    pip3 install . --user
    ```

3. 修改配置文件，配置Tair实例相关信息。编辑algos.yaml，搜索“tairvector”找到对应配置项，修改`base-args`一行，将其中的`TAIR_HOST`、`TAIR_PORT`、`TAIR_PASSWORD`三项分别替换为测试Tair实例的地址、端口和密码。参数`parallelism`为写数据和多线程查询的并发线程数，可以根据需要修改。

    ```yaml
    tairvector:
      docker-tag: ann-benchmarks-tvs
      module: ann_benchmarks.algorithms.tairvector
      constructor: TairVectorSearch
      base-args: [{"host": "TAIR_HOST", "port": "TAIR_PORT", "password": "TAIR_PASSWORD", "parallelism": 4}, "@metric", "HNSW"]
    ```

4. **[可选]** 修改要测试的参数组合。编辑algos.yaml，搜索“tairvector”找到对应配置项，“run-groups”下面就是测试的参数组合，对于HNSW索引，“arg-groups”为创建索引的参数（M和ef_construct），“query-args”为查询参数（ef_search），示例如下，可以根据需要添加或者删除。FLAT索引无任何参数，无需配置。

    ```yaml
    tairvector:
      ...
      run-groups:
        M-4:
          arg-groups:
            - {"M": 4,  "ef_construct": 500}
          query-args: [[10, 20, 40, 80, 120, 200, 400, 600, 800]]
        M-8:
          arg-groups:
            - {"M": 8,  "ef_construct": 500}
          query-args: [[10, 20, 40, 80, 120, 200, 400, 600, 800]]
    ```

5. **[可选]** 如果要测试其他算法进行对比，需要安装对应的依赖，例如hnswlib：

    ```bash
    pip3 install hnswlib==0.6.2 --user
    ```
    具体算法依赖可以参考install目录下的Dockerfile。

6. 运行测试。k、algorithm、dataset这几个参数可以按需修改。参数 runs 表示查询测试会执行5轮取最优值。

    ```bash
    # 运行单线程测试
    python3 run.py -k 10 --algorithm tairvector --dataset sift-128-euclidean --local --runs 5
    # 或者运行多线程测试
    # python3 run.py -k 10 --algorithm tairvector --dataset sift-128-euclidean --local --runs 5 --batch
    # 运行FLAT索引测试
    python3 run.py -k 10 --algorithm tairvector-flat --dataset mnist-784-euclidean --local --runs 5

    # 导出结果到csv
    python3 data_export.py --recompute --out output.csv
    # 导出多线程测试结果到csv
    # python3 data_export.py --recompute --out output.csv --batch

    # 绘制"召回率-QPS"曲线，结果在results目录
    python3 plot.py --recompute --dataset sift-128-euclidean
    ```

## 常见问题

1. 运行前会自动下载测试数据文件，必须等待下载完才能继续，否则会报错误。如果报 h5py 相关错误，可能是因为数据文件下载不完整，可以删除 data 目录下相关的数据文件再重新执行，或者手动下载数据文件放在 data 目录下。下载地址可以在[这里](https://github.com/erikbern/ann-benchmarks)找到。

2. 如果报 "Nothing to run"，说明之前已经运行过测试，需要删掉 results 目录下的结果才能重新运行。
