#!/usr/bin/env python3

import argparse
import numpy as np
import os
from multiprocessing import Process


def query_with_distances(x, train, count):
    ibits = np.sum(np.unpackbits(x & train, axis=-1), axis=-1)
    ubits = np.sum(np.unpackbits(x | train, axis=-1), axis=-1)
    # z = 0
    # for u in ubits:
    #     if u == 0:
    #         z += 1
    # if z > 0:
    #     print("WARNING ubits has %d zeros" % z)
    dists = 1.0 - (ibits.astype("float") / ubits.astype("float"))
    nearest_indices = np.argpartition(dists, count)[:count]
    return [(idx, dists[idx]) for idx in nearest_indices]


def parse_line(line):
    return np.array([int(x) for x in line.split(",")], dtype="uint8")


def load_dataset(path):
    with open(path) as f:
        data = np.array([parse_line(line) for line in f])
    return data


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        formatter_class=argparse.ArgumentDefaultsHelpFormatter
    )
    parser.add_argument("--input", type=str, help="input dir", default="sdf")
    parser.add_argument("--output", type=str, help="output dir", default="./pubchem.fp")
    parser.add_argument("-k", type=int, help="top k", default=100)
    parser.add_argument("-p", type=int, help="num of processes", default=10)
    args = parser.parse_args()

    train_set = load_dataset(os.path.join(args.input, "train.txt"))
    print("train set size: %s" % str(train_set.shape))
    test_set = load_dataset(os.path.join(args.input, "test.txt"))
    print("test set size: %s" % str(test_set.shape))

    def fn(output_dir, i, batch, train, count):
        with open(os.path.join(output_dir, "gt.%s" % str(i).zfill(3)), "w") as f:
            for x in batch:
                if sum(x) == 0:
                    print("x: ", x)
                res = query_with_distances(x, train, count)
                res.sort(key=lambda t: t[-1])
                f.write(",".join(["%d,%f" % s for s in res]) + "\n")

    process_list = []
    batch_size = int((len(test_set) + args.p - 1) / args.p)
    for i in range(args.p):
        p = Process(
            target=fn,
            args=(
                args.output,
                i,
                test_set[i * batch_size : (i + 1) * batch_size],
                train_set,
                args.k,
            ),
        )
        p.start()
        process_list.append(p)

    for p in process_list:
        p.join()
