#!/usr/bin/env python3

import argparse
import os
from functools import reduce


def parse_gt_line(line):
    fields = line.split(",")
    return [(int(fields[i]), float(fields[i + 1])) for i in range(0, len(fields), 2)]


def load_groundtruth(path):
    results = []
    with open(path) as f:
        for line in f:
            r = parse_gt_line(line)
            results.append(r)
    return results


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        formatter_class=argparse.ArgumentDefaultsHelpFormatter
    )
    parser.add_argument("--input", type=str, help="input dir", default="sdf")
    parser.add_argument("-n", type=int, help="# of partitions", default=10)
    parser.add_argument("-k", type=int, help="top k", default=100)
    args = parser.parse_args()

    query = None
    k = None

    partitions = []
    for i in range(args.n):
        p = load_groundtruth(os.path.join(args.input, str(i), "gt.txt"))
        if query is None:
            query = len(p)
        elif query != len(p):
            print("invalid data")
        if k is None:
            k = len(p[0])
        elif k != len(p[0]):
            print("invalid data")
        partitions.append(p)

    print("merging %d partitions containing %d queries, %d neighbors" % (len(partitions), query, k))
    merged = []
    for i in range(query):
        q = reduce(lambda x, y : x + y, [p[i] for p in partitions], [])
        merged.append(q)
    print(len(merged), len(merged[0]))

    results = []
    for r in merged:
        r.sort(key=lambda t: t[-1])
        results.append(r[:args.k])

    with open(os.path.join(args.input, "gt.txt"), "w") as f:
        for res in results:
            f.write(",".join(["%d,%f" % s for s in res]) + "\n")
